#include <msp.h>
#include "crystal.h"
#include "timer.h"
#include "gpio.h"
#include "fifo_function_queue.h"
#include "stdint.h"

static int ticks = 0;
static int ticks_count_limit = TICK_COUNT_LIMIT;

//Callback function for queue
void (*tick_callback)();

void initCrystal(void)
{
    //Configure LED1 as output
    P1DIR |= LED1;
    //HFXIN - HFXOUT PJ.2 and PJ.3 pin functions
    PJSEL1 =  0x0000;
    PJSEL0 = 0x0030;

    //Clock System Key Register. This unlocks CS registers
    CS -> KEY = CS_KEY_VAL;

    //Basic Clock System Control Register 1 HFXTCLK / HSMCLK-SMCLK Divider = 1
    CS->CTL1 |=CS_CTL1_DIVHS_0 + CS_CTL1_SELS_5 ;

    //Basic Clock System Control Register 2 - Use external  (>8MHz - 16 MHz) crystal for HSMCLK / capacitance of 14pF
    CS->CTL2 |=CS_CTL2_HFXTBYPASS_OFS + CS_CTL2_HFXT_EN + CS_CTL2_HFXTFREQ_2 + CS_CTL2_HFXTDRIVE;

    CS -> KEY = 0X0000FFFF;

    //Timer A Configuration - ACLK / divider of 1 / UP mode
    TIMER_A0 -> CTL |= TIMER_A_CTL_TASSEL_1 + TIMER_A_CTL_ID_0 + TIMER_A_CTL_MC_1;
    // Timer A Interrupt enable. TAIFG enabled
    TIMER_A0 -> CTL |= TIMER_A_CTL_IE;

    //Timer A Control Register - Capture compare interrupts are enabled
    TIMER_A0 -> CCTL[0] |= TIMER_A_CCTLN_CCIE;
    //Timer A Capture/Compare Register 0 - set as 250mS(1/f), being f the clock frequency. No prescaler.
    //TACCR = 250mS / (1 / 16MHz) - 1. 1 is substracted because the flag is triggered one cycle later
    TIMER_A0 -> CCR[0] = 3999;

    __NVIC_SetPriority(TA0_0_IRQn,0);
    __NVIC_EnableIRQ(TA0_0_IRQn);
}

void setTimerTickLimit(int times)
{
    ticks_count_limit = times;
    ticks = 0;
}

int getCurrentTick(void)
{
    return ticks;
}


void setCrystalISRCallbackFunction(void *callback_function)
{
    tick_callback = callback_function;
}

void TA0_0_IRQHandler(void)
{
    if(ticks == ticks_count_limit)
    {
        ticks = 0;
        P1OUT ^= LED1;
    }
    TIMER_A0->CCTL[0] &= ~TIMER_A_CCTLN_CCIFG;
    //__NVIC_DisableIRQ(TA0_0_IRQn);
    //addToFunctionQueue(tick_callback);
    //__low_power_mode_off_on_exit();
    ticks++;
}

